/*
 * LogIt 1.0
 *
 *     By:
 *          Zhodiac <zhodiac@jjf.org>
 *          - J.J.F. / Hackers Team -
 *             http://www.jjf.org 
 *
 * Date: 08/03/1999
 * License: GNU GPL
 *
 * Compiled and tested under kernels 2.0.36 and 2.2.3
 * 
 */

#include "logit.h"

 int logtype=NOLOG;
#ifndef NOGTK
 int gtk_flag=0;
 int pipe_gtk;
#endif NOGTK
 char logfile[MAXNAM];

void usage(char *prognam) {
  fprintf(stderr,"\nUsage: %s -hipsx -f <filename> -n <device>\n\n",prognam);
  fprintf(stderr,"Modes:\n");
  fprintf(stderr,"-i\t\t\tScan all incoming trafic\n");
  fprintf(stderr,"-n <device>\t\tScan all trafic on device\n\n"); 
  fprintf(stderr,"Log type:\n");
  fprintf(stderr,"-s\t\t\tUse syslog\n");
#ifndef NOGTK
  fprintf(stderr,"-f <filename>\t\tLog to filename instead of the default file\n");
  fprintf(stderr,"-x\t\t\tX-windows interface\n\n");
#else
  fprintf(stderr,"-f <filename>\t\tLog to filename instead of the default file\n\n");
#endif NOGTK
  fprintf(stderr,"Misc options:\n");
  fprintf(stderr,"-p\t\t\tPut device in Promiscuos mode\n");
  fprintf(stderr,"-h\t\t\tDisplay this help you're reading\n\n");
  exit(-1);
  }

#ifndef NOGTK
int pipe_graph(int argcf,char *argvf[])
{
 int files[2];

 if (pipe(files) < 0) {
     perror("pipe");
     exit(-1);
     }
  
    switch(fork()) {
    case 0:
      close(files[1]);
      init_graph(argcf, argvf,files[0]); 
      exit(0);
      break;
    case -1:
      perror("fork()");
      exit(-1);
      break;
    default:
      close(files[0]);
      break;     
    }
    return(files[1]);    
}
#endif NOGTK

void daemonize(void)
{
  switch(fork()) {
    case 0:
      break;
    case -1:
      perror("fork()");
      exit(-1);
      break;
    default:
      exit(0);
      break;
    } 
  
  close(STDIN_FILENO);
  close(STDOUT_FILENO);
  close(STDERR_FILENO);

  if (setsid()==-1) exit(-1);
  signal(SIGPIPE,SIG_IGN);
}

void log_it(char *buf)
{

  FILE *fp;
  time_t t;
  char *tim;

#ifndef NOGTK
  char mensaje[BUFFSIZE];
#endif NOGTK

  t = time(NULL);
  tim = ctime(&t);
  tim[strlen(tim) - 1] = '\0';
  
  if (logtype==FILELOG) {
     fp= fopen(logfile,"a+");
     fprintf(fp,"%s %s\n",tim,buf);
     fclose(fp);
     }
  else syslog(LOG_INFO,buf);

#ifndef NOGTK
  if (gtk_flag) {
     snprintf(mensaje,BUFFSIZE-1,"%s %s\n",tim,buf); 
     mensaje[BUFFSIZE-1]='\0'; 
     if (write(pipe_gtk,mensaje,strlen(mensaje))<0) gtk_flag=0;
     } 
#endif NOGTK

}

void get_sock(int proto, int file)
{
int sock,length;
struct sockaddr_in name;
char buffer[BUFFSIZE];
int aux; 
      
 switch (fork()) {
  case 0: 
     break;
  case -1:
     perror("fork");
     exit(-1);
  default:
     return; 
  }
  

 if ((sock = socket(AF_INET,SOCK_RAW,proto))<0) {
    perror("socket");
    exit(-1);
    }

 name.sin_family = AF_INET;
 name.sin_addr.s_addr = htons(INADDR_ANY);
 name.sin_port = htons(0);

 if (bind(sock, (struct sockaddr *) &name, sizeof name ) < 0) {
    perror("bind");
    exit(-1);
    }

 length = sizeof(name);
 if (getsockname(sock, (struct sockaddr *) &name, &length) < 0) {
    perror("getsockname");
    exit(-1);
    }
   
 while((aux=read(sock,buffer,BUFFSIZE))>0) write(file,buffer,aux); 

}

int incoming_sockets(void)
{ 
int file[2];  
   
 pipe(file);
 get_sock(IPPROTO_TCP,file[1]);
 get_sock(IPPROTO_UDP,file[1]);
 get_sock(IPPROTO_ICMP,file[1]);

 return(file[0]);

}

void make_promisc(int soc, char *device)
{
  struct ifreq ifr;
  
  strcpy(ifr.ifr_name,device);
  if (ioctl(soc,SIOCGIFFLAGS,&ifr)<0) {
     close(soc);
     perror("Cant get socket flags");
     exit(-1);
     }

  ifr.ifr_flags |= IFF_PROMISC;

  if (ioctl(soc,SIOCSIFFLAGS,&ifr)<0) {
     close(soc);
     perror("Cant set socket flags");
     exit(-1);
     }
}


void main(int argc, char *argv[]) {

  int sock,length;
  char buf[BUFFSIZE],device[MAXDEVICE];
  struct iphdr *ip;
  struct tcphdr *tcp;
  struct udphdr *udp;
  struct icmphdr *icmp;
  char ch, *data;
  int net_flag=0;
  int promisc_flag=0;

  printf("%s\n",PROGRAMA);
    
  strncpy(logfile,LOG,MAXNAM-1);
  logfile[MAXNAM-1]='\0';

  strncpy(program,argv[0],255);
  program[255]='\0';
 
#ifndef NOGTK
  while ((ch = getopt(argc, argv, "sxhf:n:pi")) != EOF) {
#else
  while ((ch = getopt(argc, argv, "shf:n:pi")) != EOF) {
#endif NOGTK

    switch((char)ch) {
      case 's':
        openlog(argv[0],LOG_PID,LOG_AUTHPRIV);
        logtype=SYSLOG;
        break;            
      case 'f':
        strncpy(logfile,optarg,MAXNAM-1);
        logfile[MAXNAM-1]='\0';  
        logtype=FILELOG;
        break;

#ifndef NOGTK
      case 'x': 
        gtk_flag=1;
        break;
#endif NOGTK

      case 'n':
        net_flag=1;
        strncpy(device,optarg,MAXDEVICE-1);
        device[MAXDEVICE-1]='\0'; 
        break; 
      case 'p':
        promisc_flag=1;
        break;
      case 'i':
        net_flag=0;
        break;
      case 'h':
      default:
        usage(argv[0]);
      }
    }

if (logtype==NOLOG) usage(argv[0]);

#ifndef NOGTK
 if (gtk_flag) pipe_gtk=pipe_graph(argc,argv);
#endif NOGTK

 if (net_flag){
    if ((sock = socket(AF_INET, SOCK_PACKET, htons(ETH_P_IP)))<0) {
      perror("socket");
      exit(-1);
      }
    ip=(struct iphdr *)(buf+sizeof(struct ethhdr));
    if (promisc_flag) make_promisc(sock,device);
    }
 else { 
    sock=incoming_sockets();
    ip=(struct iphdr *)buf;
    }  


  init_var();      

/* become daemon */ 
  daemonize();

  while(1) {
    if (read(sock, buf, 1024) < 0)
         {
          perror("read");
          exit(-1);
         }
 
   if ((ip->version)==4) {     
    
    length=(int)(ip->ihl<<2); 
 
    switch(ip->protocol) {
      case IPPROTO_TCP:             
             if (net_flag) { 
                tcp=(struct tcphdr *)(buf+sizeof(struct ethhdr)+length);    
                data=(char *)(buf+sizeof(struct ethhdr)+length+((tcp->th_off<<2)&0xff)); 
                }
              else {
                tcp=(struct tcphdr *)(buf+length);
                data=(char*)(buf+length+((tcp->th_off<<2)&0xff));
                }
             check_tcp(ip,tcp,data);
             break;  
      case IPPROTO_UDP:
             if (net_flag) {
                 udp=(struct udphdr *)(buf+sizeof(struct ethhdr)+length);
                 data=(char*)(buf+sizeof(struct ethhdr)+length+sizeof(struct udphdr));
                 }  
              else {
                 udp=(struct udphdr *)(buf+length);
                 data=(char*)(buf+length+sizeof(struct udphdr));
                 }
             check_udp(ip,udp,data);
             break; 
      case IPPROTO_ICMP:
             if (net_flag) {
                icmp=(struct icmphdr *)(buf+sizeof(struct ethhdr)+length);
                data=(char*)(buf+sizeof(struct ethhdr)+length+sizeof(struct icmphdr));
                }
              else {
                icmp=(struct icmphdr *)(buf+length); 
                data=(char *)(buf+length+sizeof(struct icmphdr));
                }
             check_icmp(ip,icmp,data);
             break;
      }
  }
  }
  close(sock);
}
